# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class Stack(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'org_id': 'str',
        'created_at': 'datetime',
        'events': 'list[StackEvents]'
    }

    attribute_map = {
        'id': 'id',
        'org_id': 'orgID',
        'created_at': 'createdAt',
        'events': 'events'
    }

    def __init__(self, id=None, org_id=None, created_at=None, events=None):  # noqa: E501,D401,D403
        """Stack - a model defined in OpenAPI."""  # noqa: E501
        self._id = None
        self._org_id = None
        self._created_at = None
        self._events = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if org_id is not None:
            self.org_id = org_id
        if created_at is not None:
            self.created_at = created_at
        if events is not None:
            self.events = events

    @property
    def id(self):
        """Get the id of this Stack.

        :return: The id of this Stack.
        :rtype: str
        """  # noqa: E501
        return self._id

    @id.setter
    def id(self, id):
        """Set the id of this Stack.

        :param id: The id of this Stack.
        :type: str
        """  # noqa: E501
        self._id = id

    @property
    def org_id(self):
        """Get the org_id of this Stack.

        :return: The org_id of this Stack.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this Stack.

        :param org_id: The org_id of this Stack.
        :type: str
        """  # noqa: E501
        self._org_id = org_id

    @property
    def created_at(self):
        """Get the created_at of this Stack.

        :return: The created_at of this Stack.
        :rtype: datetime
        """  # noqa: E501
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Set the created_at of this Stack.

        :param created_at: The created_at of this Stack.
        :type: datetime
        """  # noqa: E501
        self._created_at = created_at

    @property
    def events(self):
        """Get the events of this Stack.

        :return: The events of this Stack.
        :rtype: list[StackEvents]
        """  # noqa: E501
        return self._events

    @events.setter
    def events(self, events):
        """Set the events of this Stack.

        :param events: The events of this Stack.
        :type: list[StackEvents]
        """  # noqa: E501
        self._events = events

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, Stack):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other
